@@ -103,7 +103,8 @@ class Sale(Model):
         cost_line = {}
         product_ids = [line['product'] for line in values.get('lines') or []
                 if line.get('product')]
-        stockable = any(product.salable for product in product_obj.browse(product_ids))
+        stockable = any(product.type in ('stockable', 'consumable')
+            for product in product_obj.browse(product_ids))
         if cost and currency_id and stockable:
             if (values.get('currency')
                     and currency_id != values['currency']):
@@ -115,18 +116,20 @@ class Sale(Model):
             with Transaction().set_context(party_context):
                 description = product_obj.browse(product.id).rec_name
             taxes = []
-            cost_line = {
-                'type': 'line',
-                'product': product.id,
-                'description': description,
-                'quantity': 1, #XXX
-                'unit': product.sale_uom.id,
-                'unit_price': cost,
-                'shipment_cost': cost,
-                'amount': cost,
-                'taxes': taxes,
-                'sequence': 9999, #XXX
-            }
+            cost_line = sale_line_obj.default_get(
+                sale_line_obj._columns.keys())
+            cost_line.update({
+                    'type': 'line',
+                    'product': product.id,
+                    'description': description,
+                    'quantity': 1,  # XXX
+                    'unit': product.sale_uom.id,
+                    'unit_price': cost,
+                    'shipment_cost': cost,
+                    'amount': cost,
+                    'taxes': taxes,
+                    'sequence': 9999,  # XXX
+                    })
             pattern = sale_line_obj._get_tax_rule_pattern(party,
                 cost_line)
             for tax in product.customer_taxes_used:
@@ -150,7 +153,7 @@ class Sale(Model):
             values['lines'] = []
         for line in values['lines']:
             if line.get('shipment_cost'):
-                if line['shipment_cost'] != cost_line['shipment_cost']:
+                if line['shipment_cost'] != cost_line.get('shipment_cost'):
                     to_remove = line['id']
                     values['lines'].remove(line)
                     if cost_line:
